% mixture_of_gaussians_example.m
%
%  This example applies the p-voltages algorithm to a mixture of two
%  Gaussians in high dimensions.
%
%  Author: Nick Bridle, nbridle@cs.wisc.edu. January 2013.
%
%  Citation:
%    Bridle, Nick and Xiaojin Zhu. 
%    p-voltages: Laplacian Regularization for Semi-Supervised Learning on 
%      High-Dimensional Data.
%    In the Eleventh Workshop on Mining and Learning with Graphs, MLG 2013

% The dimension of the dataset
d = 15;

% Recommended p* value (Alamgir 2011)
p = d/(d-1);

mu1 = [-1.5; zeros(d-1,1)];
mu2 = [1.5; zeros(d-1,1)];
sigma = eye(d);

% Use two labeled points, one for each class
source = [-1, zeros(1,d-1)];
sink = [1, zeros(1,d-1)];

% How many points in each Gaussian
n1 = 1500;
n2 = 1500;

X = [source; sink; mvnrnd(mu1,sigma,n1); mvnrnd(mu2,sigma,n2)];
labels = [1; 0; ones(n1,1); zeros(n2,1)];
n = length(labels);

tic;
% Create a knn graph with k=log(n) and RBF weights.
bandwidth = 20;
weight = @(d)gaussian_weight(d, bandwidth);
[D, W] = create_knn_graph(X, [], weight, []);
toc;
tic;
[V,energy] = calculate_p_voltages(W, p, [1;2], [1;0]);
toc;

% Display the p-voltages graphically
% The voltages are shown on a spectrum from red to blue
% The points are projected down to three dimensions for visualization's sake
scatter3(X(:,1),X(:,2),X(:,3),5,V)
